import Head from "next/head";
import { Inter } from "next/font/google";
import {
  Alert,
  Button,
  ButtonGroup,
  ButtonToolbar,
  Col,
  Container,
  FormGroup,
  Row,
  Table,
} from "react-bootstrap";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { faCalculator, faWarning } from "@fortawesome/free-solid-svg-icons";
import { costsStructure, countUntil } from "../lib";
import { useState } from "react";
import { sprintf } from "sprintf-js";
import { useRouter } from "next/router";

const inter = Inter({ subsets: ["latin"] });

const prsDate = (dt: string, add?: number) => {
  const d = new Date(dt);
  d.setTime(
    d.getTime() + (add ?? 0) * 24 * 3600 * 1000 + d.getTimezoneOffset() * 60000
  );
  return d;
};

const fmtDate1 = (dt: Date) =>
  `${dt.toLocaleDateString("pt-BR", {
    weekday: "short",
  })}`;
const fmtDate2 = (dt: Date) =>
  `${dt.toLocaleDateString("pt-BR", {
    day: "numeric",
    month: "short",
  })}`;

export default function Home() {
  const router = useRouter();
  const dismiss = router.query["ciente"] === "sim";
  const skipdays = router.query["totais"] === "apenas";
  const notitles = router.query["titulos"] === "nao";
  const [startDate, setStartDate] = useState(costsStructure.groups.in[0]);
  const [endDate, setEndDate] = useState(
    costsStructure.groups.in[costsStructure.groups.in.length - 1]
  );
  const [apartamento, setApartamento] = useState(costsStructure.rooms[0]);
  const maxCapacity = (costsStructure.capacity as Record<string, number>)[
    apartamento
  ];
  const [hospedes, setHospedes] = useState(maxCapacity);
  if (hospedes > maxCapacity) {
    setHospedes(maxCapacity);
    return <></>;
  }
  if (endDate < startDate) {
    setStartDate(endDate);
    setEndDate(startDate);
    return <></>;
  }
  const dias = costsStructure.days.filter(
    (x) => x >= startDate && x <= endDate
  );
  const total = dias
    .map(
      (x) =>
        (costsStructure.costs as Record<string, Record<string, number>>)[x][
          apartamento
        ] * costsStructure.taxes
    )
    .reduce((x, y) => x + y);
  return (
    <>
      <Head>
        <title>Quarto BFF 2024</title>
        <meta
          name="description"
          content="Quanto deve custar meu quarto no hotel do evento durante a BFF 2024?"
        />
        <meta name="viewport" content="width=device-width, initial-scale=1" />
        <link rel="icon" href="/favicon.png" />
        <link rel="icon" href="/favicon.ico" />
      </Head>
      <Container as="main" className={inter.className}>
        {notitles || (
          <h1>
            <FontAwesomeIcon icon={faCalculator} /> Quarto BFF 2024
          </h1>
        )}
        {dismiss || (
          <>
            <Alert variant="warning">
              <FontAwesomeIcon icon={faWarning} /> Esta é uma estimativa
              não-oficial. Use-a por sua conta e risco.
            </Alert>
            <Alert variant="warning">
              <FontAwesomeIcon icon={faWarning} /> Confirme o resultado das
              contas antes de qualquer pagamento.
            </Alert>
          </>
        )}
        {notitles || <h5>Seletores</h5>}
        <Row>
          <Col sm={6}>
            <FormGroup className="mb-3">
              <h6>Apartamento</h6>
              <ButtonGroup>
                {costsStructure.rooms.map((x) => (
                  <Button
                    variant={x == apartamento ? "primary" : "outline-primary"}
                    key={x}
                    onClick={() => setApartamento(x)}
                  >
                    <big>{x}</big>
                    <br />
                    <small>
                      Até{" "}
                      {(costsStructure.capacity as Record<string, number>)[x] ??
                        1}{" "}
                      hóspedes
                    </small>
                  </Button>
                ))}
              </ButtonGroup>
            </FormGroup>
          </Col>
          <Col sm={6}>
            <FormGroup className="mb-3">
              <h6>Hóspedes</h6>
              <ButtonGroup>
                {countUntil(maxCapacity)
                  .map((x) => x + 1)
                  .map((x) => (
                    <Button
                      variant={x == hospedes ? "primary" : "outline-primary"}
                      key={x}
                      onClick={() => setHospedes(x)}
                    >
                      {x}
                    </Button>
                  ))}
              </ButtonGroup>
            </FormGroup>
          </Col>
        </Row>
        <FormGroup className="mb-3">
          <h6>Entrada (retirada das chaves, check-in, a partir das 15:00)</h6>
          <ButtonToolbar>
            {[
              costsStructure.groups.pre,
              costsStructure.groups.in,
              costsStructure.groups.pos,
            ].map((k, i) => (
              <ButtonGroup key={i} className="me-1">
                {k.map((x) => (
                  <Button
                    variant={
                      x == startDate
                        ? "primary"
                        : x > startDate && x <= endDate
                        ? "info"
                        : "outline-primary"
                    }
                    key={x}
                    onClick={() => setStartDate(x)}
                  >
                    <big>{fmtDate1(prsDate(x, 0))}</big>
                    <br />
                    <small>{fmtDate2(prsDate(x, 0))}</small>
                    <br />
                    <small>
                      <small>
                        {sprintf(
                          "R$ %.2f",
                          (
                            costsStructure.costs as Record<
                              string,
                              Record<string, number>
                            >
                          )[x][apartamento]
                        )}
                      </small>
                    </small>
                  </Button>
                ))}
              </ButtonGroup>
            ))}
          </ButtonToolbar>
        </FormGroup>
        <FormGroup className="mb-3">
          <h6>Saída (devolução das chaves, check-out, até 12:00)</h6>
          <ButtonToolbar>
            {[
              costsStructure.groups.pre,
              costsStructure.groups.in,
              costsStructure.groups.pos,
            ].map((k, i) => (
              <ButtonGroup key={i} className="me-1">
                {k.map((x) => (
                  <Button
                    variant={
                      x == endDate
                        ? "primary"
                        : x >= startDate && x < endDate
                        ? "info"
                        : "outline-primary"
                    }
                    key={x}
                    onClick={() => setEndDate(x)}
                  >
                    <big>{fmtDate1(prsDate(x, 1))}</big>
                    <br />
                    <small>{fmtDate2(prsDate(x, 1))}</small>
                  </Button>
                ))}
              </ButtonGroup>
            ))}
          </ButtonToolbar>
        </FormGroup>
        {notitles || <h5>Cálculos</h5>}
        <Table striped bordered>
          <thead>
            {skipdays || (
              <tr>
                <th colSpan={2} className="text-center">
                  Datas
                </th>
                <th className="text-center">Diária</th>
                <th className="text-center">
                  {sprintf("Taxas (%.2f%%)", (costsStructure.taxes - 1) * 100)}
                </th>
                <th className="text-center">Subtotal</th>
              </tr>
            )}
          </thead>
          <tbody>
            {skipdays ||
              dias.map((d) => (
                <tr key={d}>
                  <td className="text-center">{fmtDate2(prsDate(d, 0))}</td>
                  <td className="text-center">{fmtDate2(prsDate(d, 1))}</td>
                  <td className="text-end">
                    {sprintf(
                      "R$ %.2f",
                      (
                        costsStructure.costs as Record<
                          string,
                          Record<string, number>
                        >
                      )[d][apartamento]
                    )}
                  </td>
                  <td className="text-end">
                    {sprintf(
                      "R$ %.2f",
                      (
                        costsStructure.costs as Record<
                          string,
                          Record<string, number>
                        >
                      )[d][apartamento] *
                        (costsStructure.taxes - 1)
                    )}
                  </td>
                  <td className="text-end">
                    {sprintf(
                      "R$ %.2f",
                      (
                        costsStructure.costs as Record<
                          string,
                          Record<string, number>
                        >
                      )[d][apartamento] * costsStructure.taxes
                    )}
                  </td>
                </tr>
              ))}
            <tr>
              <th colSpan={4} className="text-end">
                Total Geral
              </th>
              <th>{sprintf("R$ %.2f", total)}</th>
            </tr>
            <tr>
              <th colSpan={4} className="text-end">
                Total dividido por Hóspede
              </th>
              <th>{sprintf("R$ %.2f", total / hospedes)}</th>
            </tr>
          </tbody>
        </Table>
        {dismiss || (
          <Alert variant="warning">
            <FontAwesomeIcon icon={faWarning} /> Esta é uma estimativa
            não-oficial. Use-a por sua conta e risco.
          </Alert>
        )}
      </Container>
    </>
  );
}
